#include "mainwindow.h"
#include <QtGui>
#include <QDateTime>
#include "smtp/CSmtp.h"
#include <iostream>

#include <errno.h>
#include <fcntl.h>
#include <linux/hiddev.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent)
{
    if ((fd = open("/dev/usb/hiddev0", 0)) == -1)
    {
	QMessageBox msgBox;
        msgBox.setText("Failed to Open the device");
        msgBox.setInformativeText("Please Make sure the device is connected\n");
        msgBox.setStandardButtons(QMessageBox::Ok);
        msgBox.exec();
    }

    index = 0;
    start_index = 0;
    Interval_msec = 1;
    email_min = 1;
    email_time = 0;
    emailFlag = true;
    maH = 100.00;
    miH = 35.00;
    maT = 40.00;
    miT = 0.00;

    shutDownEnable = false;
    serverV = QString("smtp.gmail.com");
    portV = 587;
    toV = QString("");
    fromV = QString("");
    userNameV = QString("");
    passwordV = QString("");
    QDate date = QDate::currentDate();
    QString dateString = date.toString();

    Settings();

    QTime time_1 = QTime::currentTime();
    QString timeString = time_1.toString();

    fileName = dateString + timeString + "_log";
    fileName.replace(" ","_");


    logEnable = false;

    QLabel *logo = new QLabel(this);
    logo->setPixmap(QPixmap(":/images/sensore.png"));

    QLabel *logo_1 = new QLabel(this);
    logo_1->setPixmap(QPixmap(":/images/logo.png"));

    QHBoxLayout *logoL = new QHBoxLayout;
    logoL->setContentsMargins(64,0,64,0);
    logoL->addWidget(logo_1);
    logoL->addWidget(logo);


    QPushButton *quit = new QPushButton(this);
    quit->setText("Quit");
    quit->setFixedSize(64,32);

    stop = new QPushButton(this);
    QPushButton *mail = new QPushButton(this);
    log = new QPushButton(this);

    stop->setText("STOP");
    stop->setFixedSize(64,32);


    mail->setText("SETTINGS");
    mail->setFixedSize(128,32);
    log->setText("START LOG");
    log->setFixedSize(128,32);


    QHBoxLayout *h1 = new QHBoxLayout;

    h1->addWidget(stop);
    h1->addWidget(mail);
    h1->addWidget(log);
    h1->addWidget(quit);

    QLabel *tL = new QLabel(this);
    tL->setText("TEMPERATURE");
    tL->setFixedSize(128,32);

    t = new QLineEdit(this);
    t->setFixedSize(64,32);

    QLabel *tMinL = new QLabel(this);
    tMinL->setText("MIN. TEMPERATURE");
    tMinL->setFixedSize(128,32);

    tMin = new QLineEdit(this);
    tMin->setFixedSize(64,32);

    QLabel *tMaxL = new QLabel(this);
    tMaxL->setText("MAX. TEMPERATURE");
    tMaxL->setFixedSize(128,32);

    tMax = new QLineEdit(this);
    tMax->setFixedSize(64,32);


    QLabel *hL = new QLabel(this);
    hL->setText("HUMIDITY");
    hL->setFixedSize(96,32);

    h = new QLineEdit(this);
    h->setFixedSize(64,32);

    QLabel *hMinL = new QLabel(this);
    hMinL->setText("MIN. HUMIDITY");
    hMinL->setFixedSize(96,32);

    hMin = new QLineEdit(this);
    hMin->setFixedSize(64,32);

    QLabel *hMaxL = new QLabel(this);
    hMaxL->setText("MAX. HUMIDITY");
    hMaxL->setFixedSize(96,32);

    hMax = new QLineEdit(this);
    hMax->setFixedSize(64,32);


    t->setText(QString::number(0));
    tMin->setText(QString::number(100));
    tMax->setText(QString::number(0));

    h->setText(QString::number(0));
    hMin->setText(QString::number(100));
    hMax->setText(QString::number(0));

    QGridLayout *h2 = new QGridLayout;
    //h2->setContentsMargins(128,0,128,0);
    h2->addWidget(tL,0,0);
    h2->addWidget(t,0,1);
    h2->addWidget(hL,0,2);
    h2->addWidget(h,0,3);

    h2->addWidget(tMinL,1,0);
    h2->addWidget(tMin,1,1);
    h2->addWidget(hMinL,1,2);
    h2->addWidget(hMin,1,3);

    h2->addWidget(tMaxL,2,0);
    h2->addWidget(tMax,2,1);
    h2->addWidget(hMaxL,2,2);
    h2->addWidget(hMax,2,3);

    QVBoxLayout *v = new QVBoxLayout;
    v->addLayout(logoL);
    v->addLayout(h1);
    v->addLayout(h2);
    
    QGroupBox *box = new QGroupBox(this);
    box->setLayout(v);
    QMainWindow::setCentralWidget(box);
    setFixedSize(600,300);

    timerTest();
    time = new QTimer(this);
    time->setInterval(Interval_msec*1000);
    time->start();

    time2 = new QTimer(this);
    time2->setInterval(1000);
    time2->start();

    setWindowTitle("MAGIANT");
    connect(time2,SIGNAL(timeout()),this,SLOT(timerTest2()));
    connect(time,SIGNAL(timeout()),this,SLOT(timerTest()));
    connect(stop,SIGNAL(clicked()),this,SLOT(stopTimer()));
    connect(log,SIGNAL(clicked()),this,SLOT(EnableLog()));
    connect(mail,SIGNAL(clicked()),this,SLOT(ShowSettings()));
    connect(quit,SIGNAL(clicked()),this,SLOT(close()));

}

MainWindow::~MainWindow()
{

}
void MainWindow::email()
{
    CSmtp mail;
    if((!serverV.isEmpty()) && (portV != 0))
    {
        mail.SetSMTPServer(serverV.toAscii(),portV);
        if(serverV.toAscii().contains("gmail") || serverV.toAscii().contains("yahoo"))
        {
            mail.SetSecurityType(CSmtp::USE_TLS);
        }
        else
        {
            mail.SetSecurityType(CSmtp::NO_SECURITY);
        }

    }
    if(!userNameV.isEmpty())
    {
        mail.SetLogin(userNameV.toAscii());
    }
    else
    {
        return;
    }
    if(!passwordV.isEmpty())
    {
        mail.SetPassword(passwordV.toAscii());
    }
    else
    {
        return;
    }
    if(!fromV.isEmpty())
    {
        mail.SetSenderName(fromV.toAscii());
        mail.SetSenderMail(fromV.toAscii());
        mail.SetReplyTo(fromV.toAscii());
    }
    else
    {
        return;
    }

    mail.SetSubject(subV.toAscii());
    if(!toV.isEmpty())
    {
        mail.AddRecipient(toV.toAscii());
    }

    mail.SetXPriority(XPRIORITY_NORMAL);
    mail.SetXMailer("The Bat! (v3.02) Professional");
    mail.AddMsgLine(emailText->toPlainText().toAscii());
    mail.AddMsgLine("Regards");
    mail.AddMsgLine("Magiant");
    mail.Send();
}

double MainWindow::getTemperature()
{
    int ret, i, j;
    double h,l;
    struct hiddev_report_info rinfo;
    struct hiddev_field_info finfo;
    struct hiddev_usage_ref uref;

    rinfo.report_type = HID_REPORT_TYPE_INPUT;
    rinfo.report_id = HID_REPORT_ID_FIRST;
    ret = ioctl(fd, HIDIOCGREPORTINFO, &rinfo);

    while (ret >= 0)
    {
        for (i = 0; i < rinfo.num_fields; i++)
        {
            finfo.report_id = rinfo.report_id;
            finfo.report_type= rinfo.report_type;
            finfo.field_index = i;
            ioctl(fd, HIDIOCGFIELDINFO, &finfo);
            for (j = 0; j < finfo.maxusage; j++)
            {
                 uref.report_type = rinfo.report_type;
                 uref.report_id = rinfo.report_id;
                 uref.field_index = i;
                 uref.usage_index = j;
                 ioctl(fd, HIDIOCGUCODE, &uref);
                 ioctl(fd, HIDIOCGUSAGE, &uref);
                 //printf("usage: %u\t", uref.usage_code);
                 //t->setText(QString::number(uref.usage_code));
                 if(j == 0)
                 {
                    h = uref.value;
                 }
                 else if(j == 1)
                 {
                    l = uref.value;
                 }

                 //printf("value: %u\n", uref.value);
                 //t->setText(uref.value);
                 //printf("\n");
                 }
             }

             rinfo.report_id |= HID_REPORT_ID_NEXT;
             ret = ioctl(fd, HIDIOCGREPORTINFO, &rinfo);
     }
    return (h * 256 + l)/100.0;
}




double MainWindow::getHumidity()
{
    int ret, i, j;
    double h,l;
    struct hiddev_report_info rinfo;
    struct hiddev_field_info finfo;
    struct hiddev_usage_ref uref;

    rinfo.report_type = HID_REPORT_TYPE_INPUT;
    rinfo.report_id = HID_REPORT_ID_FIRST;
    ret = ioctl(fd, HIDIOCGREPORTINFO, &rinfo);

    while (ret >= 0)
    {
        for (i = 0; i < rinfo.num_fields; i++)
        {
            finfo.report_id = rinfo.report_id;
            finfo.report_type= rinfo.report_type;
            finfo.field_index = i;
            ioctl(fd, HIDIOCGFIELDINFO, &finfo);
            for (j = 0; j < finfo.maxusage; j++)
            {
                 uref.report_type = rinfo.report_type;
                 uref.report_id = rinfo.report_id;
                 uref.field_index = i;
                 uref.usage_index = j;
                 ioctl(fd, HIDIOCGUCODE, &uref);
                 ioctl(fd, HIDIOCGUSAGE, &uref);
                 //printf("usage: %u\t", uref.usage_code);
                 //t->setText(QString::number(uref.usage_code));
                 if(j == 3)
                 {
                    h = uref.value;
                 }
                 else if(j == 4)
                 {
                    l = uref.value;
                 }

                 //printf("value: %u\n", uref.value);
                 //t->setText(uref.value);
                 //printf("\n");
                 }
             }

             rinfo.report_id |= HID_REPORT_ID_NEXT;
             ret = ioctl(fd, HIDIOCGREPORTINFO, &rinfo);
     }
    return (h * 256 + l)/100.0;
}

void MainWindow::EnableLog()
{
    if(logEnable == false)
    {
        logEnable = true;
        connect(time,SIGNAL(timeout()),this,SLOT(logData()));
        log->setText("STOP LOG");
    }
    else
    {
        log->setText("START LOG");
        logEnable = false;
    }
}
void MainWindow::logData()
{
    if(logEnable == true)
    {
        QFile outFile(fileName);
        if(!outFile.open( QIODevice::ReadWrite | QIODevice::Text | QIODevice::Append ))
        {
            qDebug() << "Failed to open file !";
            exit(1);
        };
        QTextStream out( &outFile );
        if(logEnable == true)
        {
            QDate date = QDate::currentDate();
            QTime time = QTime::currentTime();
            out <<  date.toString() +"\t" + time.toString() +"\t" + t->text() + "\t" + h->text() + "\n";
        }
        outFile.close();
    }
}

void MainWindow::stopTimer()
{
    if(time->isActive())
    {
        time->stop();
        stop->setText("START");
    }
    else
    {
        stop->setText("STOP");
        time->start();
    }
}

void MainWindow::timerTest2()
{
    email_time++;

    if(email_time >= (email_min * 60))
    {
        email_time = 0;
        emailFlag = true;
    }
    else
    {
        emailFlag = false;
    }
}
void MainWindow::timerTest()
{
    double lt = getTemperature();
    //double lt = rand() % 10;
    t->setText(QString::number(lt));

    double lh = getHumidity();
    //double lh = rand() % 100;
    h->setText(QString::number(lh));

    if(lt < tMin->text().toDouble())
    {
        tMin->setText(QString::number(lt));
    }
    if(lt > tMax->text().toDouble())
    {
        tMax->setText(QString::number(lt));
    }
    if(lh > hMax->text().toDouble())
    {
        hMax->setText(QString::number(lh));
    }
    if(lh < hMin->text().toDouble())
    {
        hMin->setText(QString::number(lh));
    }

    //if((miH > h->text().toInt()) || (maH < h->text().toInt()) || (miT > t->text().toInt()) || (maT < t->text().toInt()))
    //bool testFlag = false;
    if(miH > h->text().toDouble())
    {
        if(emailFlag == true)
        {
            emailFlag = false;
            email();
        }
        if(shutDownEnable == true)
        {
            QProcess::execute("halt");
        }
    //    qDebug() << "MiH";
    }
    else if (maH < h->text().toDouble())
    {
        if(emailFlag == true)
        {
            emailFlag = false;
            email();
        }
        if(shutDownEnable == true)
        {
            QProcess::execute("halt");
        }
    //    qDebug() << "MaH";
    }
    else if (miT > t->text().toDouble())
    {
        if(emailFlag == true)
        {
            emailFlag = false;
            email();
        }
        if(shutDownEnable == true)
        {
            QProcess::execute("halt");
        }
    }
    else if (maT < t->text().toDouble())
    {
        if(emailFlag == true)
        {
            emailFlag = false;
            email();
        }
        if(shutDownEnable == true)
        {
            QProcess::execute("halt");
        }
    }
}


void MainWindow::Settings()
{
    setDialog = new QDialog();
    QFile inFile(".settings");
    QTextStream in( &inFile );
    if(inFile.open( QIODevice::ReadOnly | QIODevice::Text))
    {
        Interval_msec = in.readLine().toInt();
        email_min = in.readLine().toInt();
        miT = in.readLine().toInt();
        maT = in.readLine().toInt();
        miH = in.readLine().toInt();
        maH = in.readLine().toInt();
        serverV = in.readLine().toAscii();
        portV = in.readLine().toInt();
        toV = in.readLine().toAscii();
        fromV = in.readLine().toAscii();
        userNameV = in.readLine().toAscii();
        passwordV = in.readLine().toAscii();
        subV = in.readLine().toAscii();
        mailV = in.readLine().toAscii();
    };

    QLabel *LtimeInt = new QLabel(setDialog);
    LtimeInt->setText("Time Interval");

    timeInt = new QLineEdit(setDialog);
    timeInt->setFixedSize(64,24);
    timeInt->setText(QString::number(Interval_msec));

    QLabel *LmailInt = new QLabel(setDialog);
    LmailInt->setText("Email Alert Interval (mins)");

    mailInt = new QLineEdit(setDialog);
    mailInt->setFixedSize(64,24);
    mailInt->setText(QString::number(email_min));

    QHBoxLayout *tLayout = new QHBoxLayout;
    tLayout->addWidget(LtimeInt);
    tLayout->addWidget(timeInt);
    tLayout->addWidget(LmailInt);
    tLayout->addWidget(mailInt);

    //tLayout->setContentsMargins(72,0,72,0);
    QLabel *LminTemp = new QLabel(setDialog);
    LminTemp->setText("Min Temp Treshold");

    minTemp = new QLineEdit(setDialog);
    minTemp->setFixedSize(64,24);
    minTemp->setText(QString::number(miT));

    QLabel *LmaxTemp = new QLabel(setDialog);
    LmaxTemp->setText("Max Temp Treshold");

    maxTemp = new QLineEdit(setDialog);
    maxTemp->setFixedSize(64,24);
    maxTemp->setText(QString::number(maT));

    QHBoxLayout *TLayout = new QHBoxLayout;
    TLayout->addWidget(LminTemp);
    TLayout->addWidget(minTemp);
    TLayout->addWidget(LmaxTemp);
    TLayout->addWidget(maxTemp);

    QLabel *LminHum = new QLabel(setDialog);
    LminHum->setText("Min Hum Treshold");

    minHum = new QLineEdit(setDialog);
    minHum->setFixedSize(64,24);
    minHum->setText(QString::number(miH));


    QLabel *LmaxHum = new QLabel(setDialog);
    LmaxHum->setText("Max Hum Treshold");


    maxHum = new QLineEdit(setDialog);
    maxHum->setFixedSize(64,24);
    maxHum->setText(QString::number(maH));

    QHBoxLayout *HLayout = new QHBoxLayout;
    HLayout->addWidget(LminHum);
    HLayout->addWidget(minHum);
    HLayout->addWidget(LmaxHum);
    HLayout->addWidget(maxHum);

    QLabel *LserverEmail = new QLabel(setDialog);
    LserverEmail->setText("SMTP Server :");
    serverEmail = new QLineEdit(setDialog);
    serverEmail->setText(serverV);

    QLabel *LportEmail = new QLabel(setDialog);
    LportEmail->setText("Port :");
    portEmail = new QLineEdit(setDialog);
    portEmail->setText(QString::number(portV));

    QLabel *LtoEmail = new QLabel(setDialog);
    LtoEmail->setText("To :");
    toEmail = new QLineEdit(setDialog);
    toEmail->setText(toV);

    QLabel *LfromEmail = new QLabel(setDialog);
    LfromEmail->setText("From :");
    fromEmail = new QLineEdit(setDialog);
    fromEmail->setText(fromV);

    QLabel *LuserName = new QLabel(setDialog);
    LuserName->setText("Login Name :");
    userName = new QLineEdit(setDialog);
    userName->setText(userNameV);

    QLabel *Lpassword = new QLabel(setDialog);
    Lpassword->setText("Password :");
    password = new QLineEdit(setDialog);
    password->setText(passwordV);
    password->setEchoMode(QLineEdit::Password);

    QLabel *subjectL = new QLabel(setDialog);
    subjectL->setText("Subject :");
    subject = new QLineEdit(setDialog);
    subject->setText(subV);

    emailText = new QTextEdit(setDialog);
    emailText->setText("Sample email text! Please edit and save before sending the mail!");



    QGridLayout *mailLayout = new QGridLayout;
    mailLayout->addWidget(LserverEmail,0,0);
    mailLayout->addWidget(serverEmail,0,1);
    mailLayout->addWidget(LportEmail,0,2);
    mailLayout->addWidget(portEmail,0,3);

    mailLayout->addWidget(LuserName,1,0);
    mailLayout->addWidget(userName,1,1);
    mailLayout->addWidget(Lpassword,1,2);
    mailLayout->addWidget(password,1,3);

    mailLayout->addWidget(LtoEmail,2,0);
    mailLayout->addWidget(toEmail,2,1);
    mailLayout->addWidget(LfromEmail,2,2);
    mailLayout->addWidget(fromEmail,2,3);

    mailLayout->addWidget(subjectL,3,0);
    mailLayout->addWidget(subject,3,1,1,3);
    mailLayout->addWidget(emailText,4,0,1,4);

    QLabel *shut = new QLabel(setDialog);
    shut->setText("Enable Shutdown");
    shutDown = new QRadioButton(setDialog);
    shutDown->setChecked(false);
    connect(shutDown,SIGNAL(toggled(bool)),this,SLOT(shutDownCall(bool)));
    mailLayout->addWidget(shut,5,1);
    mailLayout->addWidget(shutDown,5,2);

    QGroupBox *email = new QGroupBox(setDialog);
    email->setLayout(mailLayout);
    email->setCheckable(true);
    email->setTitle("Enable Email");
    email->setFixedSize(460,240);

    QPushButton *testEmail = new QPushButton(setDialog);
    testEmail->setText("TEST EMAIL");
    testEmail->setFixedWidth(96);

    QPushButton *save = new QPushButton(setDialog);
    save->setText("SAVE");

    QPushButton *cancel = new QPushButton(setDialog);
    cancel->setText("CANCEL");
    connect(cancel,SIGNAL(clicked()),setDialog,SLOT(close()));
    connect(save,SIGNAL(clicked()),this,SLOT(SaveSettings()));
    connect(testEmail,SIGNAL(clicked()),this,SLOT(sendTestEmail()));

    QHBoxLayout *saveLayout = new QHBoxLayout;
    saveLayout->addWidget(testEmail);
    saveLayout->addWidget(save);
    saveLayout->addWidget(cancel);
    saveLayout->setContentsMargins(130,0,32,0);


    QVBoxLayout *final = new QVBoxLayout;
    final->addLayout(tLayout);
    final->addLayout(TLayout);
    final->addLayout(HLayout);
    final->addWidget(email);
    final->addLayout(saveLayout);

    setDialog->setLayout(final);
    setDialog->setFixedSize(480,540);
}

void MainWindow::sendTestEmail()
{
    SaveSettings();
    email();
}
void MainWindow::ShowSettings()
{
    setDialog->show();
}
void MainWindow::shutDownCall(bool enable)
{
    shutDownEnable = enable;
}

void MainWindow::SaveSettings()
{
    Interval_msec = timeInt->text().toInt();
    time->setInterval(Interval_msec*1000);

    email_min = mailInt->text().toInt();
    miT = minTemp->text().toDouble();
    maT = maxTemp->text().toDouble();

    miH = minHum->text().toDouble();
    maH = maxHum->text().toDouble();

    serverV = serverEmail->text();
    portV = portEmail->text().toInt();
    toV = toEmail->text();
    fromV = fromEmail->text();
    userNameV = userName->text();
    passwordV = password->text();
    subV = subject->text();
    mailV = emailText->toPlainText();
    setDialog->close();

    QFile outFile(".settings");
    if(!outFile.open( QIODevice::ReadWrite | QIODevice::Text))
    {
        qDebug() << "Failed to open file !";
        exit(1);
    };


    QTextStream out1( &outFile );
    out1 << QString::number(Interval_msec) + "\n";
    out1 << QString::number(email_min) + "\n";
    out1 << QString::number(miT) + "\n";
    out1 << QString::number(maT) + "\n";
    out1 << QString::number(miH) + "\n";
    out1 << QString::number(maH) + "\n";
    out1 << serverV + "\n";
    out1 << QString::number(portV) + "\n";
    out1 << toV + "\n";
    out1 << fromV + "\n";
    out1 << userNameV + "\n";
    out1 << passwordV + "\n";
    out1 << subV + "\n";
    out1 << mailV + "\n";

    outFile.close();

}


